/*--------------------------------------------------------------------------------------------------
Stack data

stacking of processed data for implementation of stacked diff in diff. We stack following 
Cengiz, Dube, et. al (2019)

	Outline:

	1. markaz red flagging
	2. markaz red flagging with flagging history
	3. immediate markaz red flagging
	4. markaz orange flagging stacking
	5. tehsil red flagging stacking
	6. district flagging stacking (ranking)
--------------------------------------------------------------------------------------------------*/

/* stacking notes:

note 1: the stacking requires defining arbitrarily pre/post periods, thus, any change in the 
        number of periods would require a new dataset
note 2: because the stacking is based in events where units are treated, and we have treatment 
        (flagging) for each outcome, we need to build an stacked dataset for each outcome

- our main specification will use 4 pre-periods and 7 post-periods. We will obtain stacked datasets 
  for all post periods after the punishment phase
  
- we will build for each stacked dataset an indicator for threshold sample (with rd procedures)

- for other thresholds and aggregations we only use the 7 post-period dataset
*/

*---------------------------------------------------------------------------------------------------
**# 1. markaz red flagging #

*iterate over each outcome

//1. create event panel
//2. create and adjust event-panel: threshold sample, additionales variables

local treatments tpres satt fac eng math urdu // all flagging outcomes
foreach v of local treatments{
	
	*iterate over all possible post-periods
	forvalues i = 1(1)5{ 
		
		* trick for later create empty temporary file to apend all other evet-panels
		
		clear		
		gen stack = 0
		tempfile stack
		save `stack'
		
		* main data: balanced dataset
		use "$analysis/processed/cc_datapacks.dta", clear
		drop if wing == 3 // we do not consider secondary schools (not subject to monitoring)
		
		/* create time id: date version has gaps between may and september. The new variable takes 
		continuous values since 1, hence, there is no gap*/
		
		egen date_treat_e = group(date_treat) // date indicator for iterations
		
		* validate mar with double assignment of flagging
		
		gen copy = treat_`v'_lead // create a copy of the original variable
		bys mar date_treat: egen templead = nvals(treat_`v'_lead) //some obs with double accounting
		bys mar date_treat: egen temp = nvals(treat_`v'_) // original date is correct
		bys mar (date_treat `v'): gen t = treat_`v'_[_n-1] // update flagging
		bys mar date_treat (`v'): gen tord = _n // set order based on `v'
		replace t = . if tord != 1 // set to missing all except the immediatly before observation for each mar
		bys mar date_treat: egen tmax = max(t) // generalize flagging
		replace treat_`v'_lead = tmax if templead == 2 // apply only to those needed to be adjusted
		replace treat_`v'_lead = copy if templead == 2 & treat_`v'_lead == . // missing replace with original
		drop templead temp t tord tmax copy
		
		* still double accounting, basically from schools changing mar into a new mar
		* define as the majority of schools reporting (less than 1% so no sensitive)
		bys mar date_treat: egen templead = nvals(treat_`v'_lead) //some obs with double accounting
			*count ones and zeroes
		gen zero = 1 if templead == 2 & treat_`v'_lead == 0
		gen ones = 1 if templead == 2 & treat_`v'_lead == 1
		bys mar date_treat: egen numzeros = sum(zero) if templead == 2
		bys mar date_treat: egen numones = sum(ones) if templead == 2
		replace treat_`v'_lead = 0 if numzeros > numones & templead == 2
		replace treat_`v'_lead = 1 if numzeros <= numones & templead == 2
		
		drop zero ones numzeros numones templead
				
		* keep unique mar and date_treat

		*duplicates drop mar date_treat, force // this way of dropping duplicates might be unstable
		* stable solution to drop duplicates
		sort mar date_treat treat_`v'_lead mar_`v' `v'
		gen random = runiform() // generate random variable (note the seed in 0_setup)
		sort mar date_treat treat_`v'_lead mar_`v' `v' random
        bys mar date_treat (treat_`v'_lead mar_`v' `v' random): gen ord = _n // ordering
		keep if ord == 1
		drop random ord
		
		keep mar date_treat date_treat_e mar_`v' treat_`v'_lead red_`v'

		* define dates at which a treatment will be stackedators by which 
		
		gen treat_dates = date_treat_e if ((treat_`v'_lead == 1) & date_treat > tm(2011m12)) // 2011m12 first flag, but we are using as event the lead
	
		/* we define datasets for each date where a flag ocurred, so the flag date is the center of 
		each event-data. Note also that date_treat_e correspond to the relative time 1 (punish), 
		then we always will have date_treat_e - 5 to have preperiods (-1, -2, -3, -4) and 0 (flag). 
		We are only changing t based on the post periods, where the minimum is date_treat_e + 1 
		*/

		display("starting variable `v' time `i'")

		* iterate creation of event-panels 
		levelsof treat_dates
		foreach d in `r(levels)'{ 
			
			preserve
			
			*dataset between t+i to t-5 
			
			keep if ((date_treat_e <= `d' + `i') & (date_treat_e >= `d' - 5)) // keep desired time span
			
			*drop if mar outcome is empty
			
			drop if missing(mar_`v')
						
			*drop units that do not have t and t-1
			
			bys mar: egen num_mar = count(mar) //count number of times mar appear
			gen num_periods = 6 + `i' // markaz that appear consecutively 6 + i periods
			keep if num_mar == num_periods // only mars that appear all the time span
			drop num_periods num_mar
						
			*conditional if exist data
			
			if(_N > 0){
				
				bys mar (date_treat): gen time_id = _n // create time id
					
				/* we must eliminate mar treated in t-n: in such case, a unit would be treated 
				twice (in t-1 and t) or would be treated only on t-1, in which case would not be 
				a clean control
				*/
				
				gen before = (time_id < 6)  // goes from 1 to 6 + `i', so always drop treated in < 6
				sort mar date_treat // sort by date within mar
				bys mar (date_treat): egen treated_before = max(treat_`v'_lead) if (before == 1) // treated before time 6
				bys mar: egen max_before = max(treated_before)
				drop if max_before == 1 // drop if treated before
				drop treated_before max_before
					
				*if there still exist data: dropping treated before might lead to no-data for event
				if(_N > 0){
						
					/*if treated in t+1 but not in t, we must drop, as would not be treated in 
					the main event data we are centering the event-data
					*/
						
					*treated in t+1
					gen stag1 = ((treat_`v'_lead == 1) & (time_id > 6))
						
					*treated in t
					gen stag2 = ((treat_`v'_lead == 1) & (time_id == 6))
						
					*generalizing to all units
					
					bys mar: egen sum_stag1 = max(stag1)
					bys mar: egen sum_stag2 = max(stag2)

					*drop not treated in main event
					
					gen drop_units = (sum_stag2 == 0 & sum_stag1 == 1) // not treated in t, but treated in t+1
					drop if drop_units == 1
					drop before stag1 stag2 sum_stag1 sum_stag2 drop_units
						
					*stack unique id
					
					gen stack = `d'
					drop time_id
					bys mar (date_treat): gen time_id = _n // readjust timeid
					
					
				} // end if statement
				else if(_N == 0){ // if there is no data after dropping the not clean controls
						
					*display if no observations
					display "`d' have 0 observations"
						
				} // end else statement 
			
			} // end if statement
			else if(_N == 0){ // if there is no data for the particular event date
				
				*display if no observations
				display "`d' have 0 observations"

			} // end else statement
			
			*append evet-panel datatsets
			append using `stack' // append to original stack! then for next iterations we rewritte
			tempfile stack
			save `stack'
				
			restore // recover original data to create the next event time panel
		
		} // end of loop statement for event panels of outcomes
		
		* use temporary stacked dataset
		
		use `stack', clear
		
		* drop "temporary file stack"

		drop if stack == 0

		* compress for memory purposes

		compress

		* relative time: normalize to 0 the flag period (time_id == 5)
		
		gen relativetime = time_id - 5

		/* create constant treatment variable: if flag_period = 1 in relativetime == 1 to validate, 
		recreate main treatment
		*/
		
		bys stack mar: gen temp_lead = ((treat_`v'_lead == 1) & (time_id == 6) & (date_treat > tm(2011m12)))
		bys stack mar: egen flagged = max(temp_lead) //if ever flagged, then treated
		drop temp_lead
		drop if missing(flagged) // drop all the observations that do not report data en the flagging period
		
		* post-periods
		
		//after punish period
		gen postpunish = (relativetime > 0)
		//after flag period
		gen postflag = (relativetime > -1)
 
		/* regression discontinuity samples:
		
		we need a tighter comparisson of treated and control units, then we use an optimal bandwidth 
		to define for each event-dataset the group of markaz that are around the threshold
		*/
		
		*define thresholds: scores have the same threshold and for student attendance need to defined it separately
		
		if("`v'" == "tpres"){
			local thres = 86
			local min = 86
		}
		else if("`v'" == "fac"){
			local thres = 90
			local min = 87 // for functional facilities there is less variation, then being close to 90 is not enough
		}
		else{
			local thres = 70
			local min = 70
		}
		
		*run rd 
		
		if("`v'" != "satt"){
			
			preserve
			
			*keep event mar relative time and variable of interest
			
			keep stack mar relativetime mar_`v'
			
			/*for the rd we need to compare the flagging period (relative time 0) with the period of 
			the punishment (relative time 1). Then we create the lag of the markaz variable for 1, 
			to have both values of 1 and 0 in the  same row*/
			
			gen temp = mar_`v' if relativetime == 0
			bys stack mar (relativetime): egen mar_`v'_lag0 = max(temp) // period 0 outcome
			drop temp
			
			*keep only the iterest relative time
			
			keep if relativetime == 1 // so compare outcome in 1 with just created outcome in 0
			drop relativetime
			
			*create variables to populate

			gen bw1 = .
			gen bias1 = .
			
			* estimate rd: we have to iterate over each event dataset
			
			levelsof stack // all possible event-dates 
			foreach d in `r(levels)'{
				
				display `d'
				qui sum mar_`v'_lag0 if stack == `d'
				*conditions to ensure if threshold sample exist

				if((`r(min)' < `min' & `r(N)' > 50 & "`v'" == "tpres"             ) | /// general conditions
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "fac"  & `d'   != 32) | /// general conditions
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "eng"  & `d'   != 65) | /// general conditions
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "math" & `d'   != 65) | /// conditions for math
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "urdu" & `d'   != 65)   /// conditions for urdu
				  ){ // set up conditions for events that have enough obs (fix some urdu problems)
					
					*run the optimal bandwidth selection: Cattaneo and Calonico

					rdbwselect mar_`v' mar_`v'_lag0 if stack == `d', ///
						c(`thres') p(2) q(3) kernel(triangular) bwselect(mserd) vce(cluster mar) all

						if(`e(h_mserd)' < `e(b_mserd)'){ //if effectively exist a bandwidth
						
						*calculate auxiliary variables
						
						replace bw1 = `e(h_cercomb1)' if stack == `d' // msecomb1 cercomb1
						replace bias1 = `e(b_cercomb1)' if stack == `d'
						
					} // end of if statement of existance of bandwidth
				
				} // end of if statament of existance of rd for each stack

			} // end of loop over stacks for rd procedure
			
			* variables for rd samples	
			
			gen rd1 = (mar_`v'_lag0 >= `thres' - bw1 & mar_`v'_lag0 <= `thres' + bw1 & !missing(bw1)) 
			gen biasrd1 = (mar_`v'_lag0 >= `thres' - bias1 & mar_`v'_lag0 <= `thres' + bias1 & !missing(bw1))

			
		} //  end if statement for all except student attendance
		else{ // because student attendance threshold is not constant, need to recenter to 0	
		
			/*for student attendance the thresholds change in time, then we have to normalize 
			threshold to 0*/
			
			preserve
			
			*keep event mar relative time and variable of interest
			
			keep stack mar relativetime mar_`v' red_`v'
			
			/*for the rd we need to compare the flagging period (relative time 0) with the period of 
			the punishment (relative time 1). Then we create the lag of the markaz variable for 1, 
			to have both values of 1 and 0 in the  same row*/
			
			gen temp = mar_`v' if relativetime == 0
			bys stack mar (relativetime): egen mar_`v'_lag0 = max(temp) // period 0 outcome
			drop temp
			
			*keep only the iterest relative time
			
			keep if relativetime == 1 // so compare outcome in 1 with just created outcome in 0
			drop relativetime
			
			*create variables to populate
			
			gen bw1 = .
			gen bias1 = .

			* need to center as in satt thresholds are not constant
			
			gen c_mar_`v' = mar_`v' - red_`v'
			gen c_mar_`v'_lag0 = mar_`v'_lag0 - red_`v'
			
			* estimate rd: we have to iterate over each event dataset
			
			levelsof stack // all possible event-dates 
			foreach d in `r(levels)'{

				qui sum c_mar_`v'_lag0 if stack == `d'
				if(`r(min)' < -2 &  `r(N)' > 50 ){ // from 0 to -2 to avoid conformability error
					
					*run the optimal bandwidth selection: Cattaneo and Calonico
					
					rdbwselect c_mar_`v' c_mar_`v'_lag0 if stack == `d', ///
						c(0) p(2) q(3) kernel(triangular) bwselect(mserd) vce(cluster mar) all
						
					if(`e(h_mserd)' < `e(b_mserd)'){
						
						*calculate auxiliary variables
						replace bw1 = `e(h_cercomb1)' if stack == `d'
						replace bias1 = `e(b_cercomb1)' if stack == `d'
						
					} // end of if statement of existance of bandwidth
					
				} // end of if statament of existance of rd for each stack

			} // end of loop over stacks for rd procedure
			
			*variables for samples	
			
			gen rd1 = (c_mar_`v'_lag0 >= 0 - bw1 & c_mar_`v'_lag0 <= 0 + bw1 & !missing(bw1)) 
			gen biasrd1 = (c_mar_`v'_lag0 >= 0 - bias1 & c_mar_`v'_lag0 <= 0 + bias1 & !missing(bw1))
		
		} //  end else statement for student attendance
		
		* save tempfile
		
		keep stack mar rd1 biasrd1 bw1 bias1
		tempfile rd_sample
		save `rd_sample'
		restore //note the preserve for this restore is within the rd procedure
		
		*merge it with the original stacked data
		
		merge m:1 stack mar using `rd_sample'

		* keep desired observations
		
		keep if _merge == 3
		drop _merge

		* merge with original: all schools per mar and date

		joinby mar date_treat using "$analysis/processed/cc_datapacks.dta"
		
		* mean teachers per markaz
		gen temp = totalteachers if relativetime < 0
		bys stack mar: egen avg_totalreportedteachers_mar = mean(temp) 
		drop temp

		* save
		compress
		save "$analysis/processed/stacked/cc_stackdata`i'_`v'.dta", replace // i = post period, v = outcome
	
		display("finish variable `v' with post periods `i'")

	} // end loop statement for post period i
	
	*after the last `i' post period, it starts again for the next variable v
	display("finish variable `v', pasing to the next one")

} // end loop statement for all variables

*---------------------------------------------------------------------------------------------------
**# 2. markaz red flagging with flagging history #

/* we allow to units to be treated before our main event, so we control by flagging history fixed 
effects
*/

*iterate over each outcome

//1. create event panel
//2. create and adjust event-panel: threshold sample, additionales variables

local treatments tpres satt fac eng math urdu // all flagging outcomes
foreach v of local treatments{
	
	*iterate over all possible post-periods
	
	forvalues i = 5(1)5{ // only run for the full set of post periods
		
		* trick for later create empty temporary file to apend all other evet-panels
		
		clear		
		gen stack = 0
		tempfile stack
		save `stack'
		
		* main data: balanced dataset
		
		use "$analysis/processed/cc_datapacks.dta", clear
		drop if wing == 3 // we do not consider secondary schools (not subject to monitoring)
		
		/* create time id: date version has gaps between may and september. The new variable takes 
		continuous values since 1, hence, there is no gap
		*/
		
		egen date_treat_e = group(date_treat) // date indicator for iterations

		* validate mar with double assignment of flagging
		
		gen copy = treat_`v'_lead // create a copy of the original variable
		bys mar date_treat: egen templead = nvals(treat_`v'_lead) //some obs with double accounting
		bys mar date_treat: egen temp = nvals(treat_`v'_) // original date is correct
		bys mar (date_treat `v'): gen t = treat_`v'_[_n-1] // update flagging
		bys mar date_treat (`v'): gen tord = _n // set order based on `v'
		replace t = . if tord != 1 // set to missing all except the immediatly before observation for each mar
		bys mar date_treat: egen tmax = max(t) // generalize flagging
		replace treat_`v'_lead = tmax if templead == 2 // apply only to those needed to be adjusted
		replace treat_`v'_lead = copy if templead == 2 & treat_`v'_lead == . // missing replace with original
		drop templead temp t tord tmax copy
		
		* still double accounting, basically from schools changing mar into a new mar
		* define as the majority of schools reporting (less than 1% so no sensitive)
		bys mar date_treat: egen templead = nvals(treat_`v'_lead) //some obs with double accounting
			*count ones and zeroes
		gen zero = 1 if templead == 2 & treat_`v'_lead == 0
		gen ones = 1 if templead == 2 & treat_`v'_lead == 1
		bys mar date_treat: egen numzeros = sum(zero) if templead == 2
		bys mar date_treat: egen numones = sum(ones) if templead == 2
		replace treat_`v'_lead = 0 if numzeros > numones & templead == 2
		replace treat_`v'_lead = 1 if numzeros <= numones & templead == 2
		
		drop zero ones numzeros numones templead
				
		* keep unique mar and date_treat

		*duplicates drop mar date_treat, force // this way of dropping duplicates might be unstable
		* stable solution to drop duplicates
		sort mar date_treat treat_`v'_lead
		gen random = runiform() // generate random variable (note the seed in 0_setup)
		sort mar date_treat treat_`v'_lead random
        bys mar date_treat: gen ord = _n // ordering
		keep if ord == 1
		drop random ord
		
		keep mar date_treat date_treat_e mar_`v' treat_`v'_lead red_`v'

		* define dates at which a treatment will be stackedators by which 

		gen treat_dates = date_treat_e if ((treat_`v'_lead == 1) & date_treat > tm(2011m12)) // 2011m12 first flag, but we are using as event the lead

		/* we define datasets for each date where a flag ocurred, so the flag date is the center of 
		each event-data. Note also that date_treat_e correspond to the relative time 1 (punish), 
		then we always will have date_treat_e - 5 to have preperiods (-1, -2, -3, -4) and 0 (flag). 
		We are only changing t based on the post periods, where the minimum is date_treat_e + 1 
		*/

		display("starting variable `v' time `i'")

		* iterate creation of event-panels 
		levelsof treat_dates
		foreach d in `r(levels)'{ 
			
			preserve
			
			*dataset between t+i to t-5 
			
			keep if ((date_treat_e <= `d' + `i') & (date_treat_e >= `d' - 5)) // keep desired time span
			
			*drop if mar outcome is empty
			
			drop if missing(mar_`v')
			
			*adjust treatment in case of missings
			
			bys mar (date_treat): replace treat_`v'_lead = treat_`v'_[_n-1] if missing(treat_`v'_lead)
			
			*drop units that do not have t and t-1
			
			bys mar: egen num_mar = count(mar) //count number of times mar appear
			gen num_periods = 6 + `i' // markaz that appear consecutively 6 + i periods
			keep if num_mar == num_periods // only mars that appear all the time span
			drop num_periods num_mar

			*conditional if exist data
			
			if(_N > 0){
				
				bys mar (date_treat): gen time_id = _n // create time id
					
				/* in the previous section we droppe units treated before our main event, here we 
				keep them and allow for flagging history fixed effects
				*/
									
				*if there still exist data: dropping treated before might lead to no-data for event
				if(_N > 0){
						
					/*if treated in t+1 but not in t, we must drop, as would not be treated in 
					the main event data we are centering the event-data
					*/
						
					*treated in t+1
					gen stag1 = ((treat_`v'_lead == 1) & (time_id > 6))
						
					*treated in t
					gen stag2 = ((treat_`v'_lead == 1) & (time_id == 6))
						
					*generalizing to all units
					
					bys mar: egen sum_stag1 = max(stag1)
					bys mar: egen sum_stag2 = max(stag2)

					*drop not treated in main event
					
					gen drop_units = (sum_stag2 == 0 & sum_stag1 == 1) // not treated in t, but treated in t+1
					drop if drop_units == 1
					drop stag1 stag2 sum_stag1 sum_stag2 drop_units
						
					*stack unique id
					
					gen stack = `d'
					drop time_id
					bys mar (date_treat): gen time_id = _n // readjust timeid
					
					
				} // end if statement
				else if(_N == 0){ // if there is no data after dropping the not clean controls
						
					*display if no observations
					display "`d' have 0 observations"
						
				} // end else statement 
			
			} // end if statement
			else if(_N == 0){ // if there is no data for the particular event date
				
				*display if no observations
				display "`d' have 0 observations"

			} // end else statement
			
			*append evet-panel datatsets
			append using `stack' // append to original stack! then for next iterations we rewritte
			tempfile stack
			save `stack'
				
			restore // recover original data to create the next event time panel
		
		} // end of loop statement for event panels of outcomes
		
		* use temporary stacked dataset
		
		use `stack', clear
		
		* drop "temporary file stack"

		drop if stack == 0

		* compress for memory purposes

		compress

		* relative time: normalize to 0 the flag period (time_id == 5)
		
		gen relativetime = time_id - 5

		/* create constant treatment variable: if flag_period = 1 in relativetime == 1 to validate, 
		recreate main treatment
		*/
		
		bys stack mar: gen temp_lead = ((treat_`v'_lead == 1) & (time_id == 6))
		bys stack mar: egen flagged = max(temp_lead) //if ever flagged, then treated
		drop temp_lead
		drop if missing(flagged) // drop all the observations that do not report data en the flagging period

		* post-periods
		
		//after punish period
		gen postpunish = (relativetime > 0)
		//after flag period
		gen postflag = (relativetime > -1)
 
		/* regression discontinuity samples:
		
		we need a tighter comparisson of treated and control units, then we use an optimal bandwidth 
		to define for each event-dataset the group of markaz that are around the threshold
		*/
		
		*define thresholds: scores have the same threshold and for student attendance need to defined it separately
		
		if("`v'" == "tpres"){
			local thres = 86
			local min = 86
		}
		else if("`v'" == "fac"){
			local thres = 90
			local min = 87 // for functional facilities there is less variation, then being close to 90 is not enough
		}
		else{
			local thres = 70
			local min = 70
		}
		
		*run rd 
		
		if("`v'" != "satt"){
			
			preserve
			
			*keep event mar relative time and variable of interest
			
			keep stack mar relativetime mar_`v'
			
			/*for the rd we need to compare the flagging period (relative time 0) with the period of 
			the punishment (relative time 1). Then we create the lag of the markaz variable for 1, 
			to have both values of 1 and 0 in the  same row*/
			
			gen temp = mar_`v' if relativetime == 0
			bys stack mar (relativetime): egen mar_`v'_lag0 = max(temp) // period 0 outcome
			drop temp
			
			*keep only the iterest relative time
			
			keep if relativetime == 1 // so compare outcome in 1 with just created outcome in 0
			drop relativetime
			
			*create variables to populate
			
			gen bw1 = .
			gen bias1 = .
			
			* estimate rd: we have to iterate over each event dataset
			
			levelsof stack // all possible event-dates 
			foreach d in `r(levels)'{
				
				display `d'
				qui sum mar_`v'_lag0 if stack == `d'
				*conditions to ensure if threshold sample exist

				if((`r(min)' < `min' & `r(N)' > 50 & "`v'" == "tpres"             )     | /// general conditions
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "fac"               )     | /// general conditions
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "eng"  & `d'   != 65)     | /// conditions for math
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "math" & `d'   != 65)     | /// conditions for math
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "urdu" & `d'   != 65)       /// conditions for urdu
				  ){ // set up conditions for events that have enough obs (fix some urdu problems)
					
					*run the optimal bandwidth selection: Cattaneo and Calonico
					
					rdbwselect mar_`v' mar_`v'_lag0 if stack == `d', ///
						c(`thres') p(2) q(3) kernel(triangular) bwselect(mserd) vce(cluster mar) all
					
					if(`e(h_mserd)' < `e(b_mserd)'){ //if effectively exist a bandwidth
						
						*calculate auxiliary variables
						
						replace bw1 = `e(h_cercomb1)' if stack == `d'
						replace bias1 = `e(b_cercomb1)' if stack == `d'
						
					} // end of if statement of existance of bandwidth
				
				} // end of if statament of existance of rd for each stack

			} // end of loop over stacks for rd procedure
			
			* variables for rd samples	
			
			gen rd1 = (mar_`v'_lag0 >= `thres' - bw1 & mar_`v'_lag0 <= `thres' + bw1 & !missing(bw1)) 
			gen biasrd1 = (mar_`v'_lag0 >= `thres' - bias1 & mar_`v'_lag0 <= `thres' + bias1 & !missing(bw1))

			
		} //  end if statement for all except student attendance
		else{ // because student attendance threshold is not constant, need to recenter to 0	
		
			/*for student attendance the thresholds change in time, then we have to normalize 
			threshold to 0*/
			
			preserve
			
			*keep event mar relative time and variable of interest
			
			keep stack mar relativetime mar_`v' red_`v'
			
			/*for the rd we need to compare the flagging period (relative time 0) with the period of 
			the punishment (relative time 1). Then we create the lag of the markaz variable for 1, 
			to have both values of 1 and 0 in the  same row*/
			
			gen temp = mar_`v' if relativetime == 0
			bys stack mar (relativetime): egen mar_`v'_lag0 = max(temp) // period 0 outcome
			drop temp
			
			*keep only the iterest relative time
			
			keep if relativetime == 1 // so compare outcome in 1 with just created outcome in 0
			drop relativetime
			
			*create variables to populate
			
			gen bw1 = .
			gen bias1 = .

			* need to center as in satt thresholds are not constant
			
			gen c_mar_`v' = mar_`v' - red_`v'
			gen c_mar_`v'_lag0 = mar_`v'_lag0 - red_`v'
			
			* estimate rd: we have to iterate over each event dataset
			
			levelsof stack // all possible event-dates 
			foreach d in `r(levels)'{

				qui sum c_mar_`v'_lag0 if stack == `d'
				if(`r(min)' < -2 &  `r(N)' > 50 ){ // from 0 to -2 to avoid conformability error
					
					*run the optimal bandwidth selection: Cattaneo and Calonico
					
					rdbwselect c_mar_`v' c_mar_`v'_lag0 if stack == `d', ///
						c(0) p(2) q(3) kernel(triangular) bwselect(mserd) vce(cluster mar) all
						
					if(`e(h_mserd)' < `e(b_mserd)'){
						
						*calculate auxiliary variables
						replace bw1 = `e(h_cercomb1)' if stack == `d'
						replace bias1 = `e(b_cercomb1)' if stack == `d'
						
					} // end of if statement of existance of bandwidth
					
				} // end of if statament of existance of rd for each stack

			} // end of loop over stacks for rd procedure
			
			*variables for samples	
			
			gen rd1 = (c_mar_`v'_lag0 >= 0 - bw1 & c_mar_`v'_lag0 <= 0 + bw1 & !missing(bw1)) 
			gen biasrd1 = (c_mar_`v'_lag0 >= 0 - bias1 & c_mar_`v'_lag0 <= 0 + bias1 & !missing(bw1))
		
		} //  end else statement for student attendance
		
		* save tempfile
		
		keep stack mar rd1 biasrd1 bw1 bias1
		tempfile rd_sample
		save `rd_sample'
		restore //note the preserve for this restore is within the rd procedure
		
		*merge it with the original stacked data
		
		merge m:1 stack mar using `rd_sample'
		
		* keep desired observations
		
		keep if _merge == 3
		drop _merge

		* merge with original: all schools per mar and date

		joinby mar date_treat using "$analysis/processed/cc_datapacks.dta"

		* mean teachers per markaz
		gen temp = totalteachers if relativetime < 0
		bys stack mar: egen avg_totalreportedteachers_mar = mean(temp) 
		drop temp

		* save
	
		save "$analysis/processed/stacked/cc_stackdata`i'_`v'_flaghist.dta", replace // i = post period, v = outcome
	
		display("finish variable `v' with post periods `i'")

	} // end loop statement for post period i
	
	*after the last `i' post period, it starts again for the next variable v
	display("finish variable `v', pasing to the next one")

} // end loop statement for all variables

*---------------------------------------------------------------------------------------------------
**# 3. immediate markaz red flagging #

* we do not consider any pre period before the flagging

*iterate over each outcome

//1. create event panel
//2. create and adjust event-panel: threshold sample, additionales variables

local treatments tpres satt fac eng math urdu // all flagging outcomes
foreach v of local treatments{
	
	*iterate over all possible post-periods
	
	forvalues i = 0(5)5{ // only run for the full set of post periods and (no) post periods
		
		* trick for later create empty temporary file to apend all other evet-panels
		
		clear		
		gen stack = 0
		tempfile stack
		save `stack'
		
		* main data: balanced dataset
		
		use "$analysis/processed/cc_datapacks.dta", clear
		drop if wing == 3 // we do not consider secondary schools (not subject to monitoring)
		
		/* create time id: date version has gaps between may and september. The new variable takes 
		continuous values since 1, hence, there is no gap
		*/
		
		egen date_treat_e = group(date_treat) // date indicator for iterations

		* validate mar with double assignment of flagging
		
		gen copy = treat_`v'_lead // create a copy of the original variable
		bys mar date_treat: egen templead = nvals(treat_`v'_lead) //some obs with double accounting
		bys mar date_treat: egen temp = nvals(treat_`v'_) // original date is correct
		bys mar (date_treat `v'): gen t = treat_`v'_[_n-1] // update flagging
		bys mar date_treat (`v'): gen tord = _n // set order based on `v'
		replace t = . if tord != 1 // set to missing all except the immediatly before observation for each mar
		bys mar date_treat: egen tmax = max(t) // generalize flagging
		replace treat_`v'_lead = tmax if templead == 2 // apply only to those needed to be adjusted
		replace treat_`v'_lead = copy if templead == 2 & treat_`v'_lead == . // missing replace with original
		drop templead temp t tord tmax copy
		
		* still double accounting, basically from schools changing mar into a new mar
		* define as the majority of schools reporting (less than 1% so no sensitive)
		bys mar date_treat: egen templead = nvals(treat_`v'_lead) //some obs with double accounting
			*count ones and zeroes
		gen zero = 1 if templead == 2 & treat_`v'_lead == 0
		gen ones = 1 if templead == 2 & treat_`v'_lead == 1
		bys mar date_treat: egen numzeros = sum(zero) if templead == 2
		bys mar date_treat: egen numones = sum(ones) if templead == 2
		replace treat_`v'_lead = 0 if numzeros > numones & templead == 2
		replace treat_`v'_lead = 1 if numzeros <= numones & templead == 2
		
		drop zero ones numzeros numones templead
				
		* keep unique mar and date_treat

		*duplicates drop mar date_treat, force // this way of dropping duplicates might be unstable
		* stable solution to drop duplicates
		sort mar date_treat treat_`v'_lead
		gen random = runiform() // generate random variable (note the seed in 0_setup)
		sort mar date_treat treat_`v'_lead random
        bys mar date_treat: gen ord = _n // ordering
		keep if ord == 1
		drop random ord

		keep mar date_treat date_treat_e mar_`v' treat_`v'_lead red_`v'

		* define dates at which a treatment will be stackedators by which 

		gen treat_dates = date_treat_e if ((treat_`v'_lead == 1) & date_treat > tm(2011m12)) // 2011m12 first flag, but we are using as event the lead

		/* we define datasets for each date where a flag ocurred, so the flag date is the center of 
		each event-data. Note also that date_treat_e correspond to the relative time 1 (punish), 
		then we always will have date_treat_e - 5 to have preperiods (-1, -2, -3, -4) and 0 (flag). 
		We are only changing t based on the post periods, where the minimum is date_treat_e + 1 
		*/

		display("starting variable `v' time `i'")

		* iterate creation of event-panels 
		levelsof treat_dates
		foreach d in `r(levels)'{ 
			
			preserve
			
			*dataset between t+i to t-5 
			
			keep if ((date_treat_e <= `d' + `i') & (date_treat_e >= `d' - 1)) // keep desired time span
			
			*drop if mar outcome is empty
			
			drop if missing(mar_`v')
			
			*adjust treatment in case of missings
			
			bys mar (date_treat): replace treat_`v'_lead = treat_`v'_[_n-1] if missing(treat_`v'_lead)
			
			*drop units that do not have t and t-1
			
			bys mar: egen num_mar = count(mar) //count number of times mar appear
			gen num_periods = 2 + `i' // markaz that appear consecutively 6 + i periods
			keep if num_mar == num_periods // only mars that appear all the time span
			drop num_periods num_mar

			*conditional if exist data
			
			if(_N > 0){
				
				bys mar (date_treat): gen time_id = _n // create time id
					
				/* in the previous section we droppe units treated before our main event, here we 
				keep them and allow for flagging history fixed effects
				*/
									
				*if there still exist data: dropping treated before might lead to no-data for event
				if(_N > 0){
						
					/*if treated in t+1 but not in t, we must drop, as would not be treated in 
					the main event data we are centering the event-data
					*/
						
					*treated in t+1
					gen stag1 = ((treat_`v'_lead == 1) & (time_id > 2))
						
					*treated in t
					gen stag2 = ((treat_`v'_lead == 1) & (time_id == 2))
						
					*generalizing to all units
					
					bys mar: egen sum_stag1 = max(stag1)
					bys mar: egen sum_stag2 = max(stag2)

					*drop not treated in main event
					
					gen drop_units = (sum_stag2 == 0 & sum_stag1 == 1) // not treated in t, but treated in t+1
					drop if drop_units == 1
					drop stag1 stag2 sum_stag1 sum_stag2 drop_units
						
					*stack unique id
					
					gen stack = `d'
					drop time_id
					bys mar (date_treat): gen time_id = _n // readjust timeid
					
					
				} // end if statement
				else if(_N == 0){ // if there is no data after dropping the not clean controls
						
					*display if no observations
					display "`d' have 0 observations"
						
				} // end else statement 
			
			} // end if statement
			else if(_N == 0){ // if there is no data for the particular event date
				
				*display if no observations
				display "`d' have 0 observations"

			} // end else statement
			
			*append evet-panel datatsets
			append using `stack' // append to original stack! then for next iterations we rewritte
			tempfile stack
			save `stack'
				
			restore // recover original data to create the next event time panel
		
		} // end of loop statement for event panels of outcomes
		
		* use temporary stacked dataset
		
		use `stack', clear
		
		* drop "temporary file stack"

		drop if stack == 0

		* compress for memory purposes

		compress

		* relative time: normalize to 0 the flag period (time_id == 5)
		
		gen relativetime = time_id - 1

		/* create constant treatment variable: if flag_period = 1 in relativetime == 1 to validate, 
		recreate main treatment
		*/
		
		bys stack mar: gen temp_lead = ((treat_`v'_lead == 1) & (time_id == 2))
		bys stack mar: egen flagged = max(temp_lead) //if ever flagged, then treated
		drop temp_lead
		drop if missing(flagged) // drop all the observations that do not report data en the flagging period

		* post-periods
		
		//after punish period
		gen postpunish = (relativetime > 0)
 
		/* regression discontinuity samples:
		
		we need a tighter comparisson of treated and control units, then we use an optimal bandwidth 
		to define for each event-dataset the group of markaz that are around the threshold
		*/
		
		*define thresholds: scores have the same threshold and for student attendance need to defined it separately
		
		if("`v'" == "tpres"){
			local thres = 86
			local min = 86
		}
		else if("`v'" == "fac"){
			local thres = 90
			local min = 87 // for functional facilities there is less variation, then being close to 90 is not enough
		}
		else{
			local thres = 70
			local min = 70
		}
		
		*run rd 
		
		if("`v'" != "satt"){
			
			preserve
			
			*keep event mar relative time and variable of interest
			
			keep stack mar relativetime mar_`v'
			
			/*for the rd we need to compare the flagging period (relative time 0) with the period of 
			the punishment (relative time 1). Then we create the lag of the markaz variable for 1, 
			to have both values of 1 and 0 in the  same row*/
			
			gen temp = mar_`v' if relativetime == 0
			bys stack mar (relativetime): egen mar_`v'_lag0 = max(temp) // period 0 outcome
			drop temp
			
			*keep only the iterest relative time
			
			keep if relativetime == 1 // so compare outcome in 1 with just created outcome in 0
			drop relativetime
			
			*create variables to populate
			
			gen bw1 = .
			gen bias1 = .
			
			* estimate rd: we have to iterate over each event dataset
			
			levelsof stack // all possible event-dates 
			foreach d in `r(levels)'{
				
				display `d'
				qui sum mar_`v'_lag0 if stack == `d'
				*conditions to ensure if threshold sample exist
				
				if((`r(min)' < `min' & `r(N)' > 50 & "`v'" == "tpres"                       )     | /// general conditions
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "fac"                         )     | /// general conditions
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "eng"  & `d' != 65            )     | /// general conditions
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "math" & `d' != 46 & `d' != 65)     | /// conditions for math
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "urdu" & `d' != 65            )       /// conditions for urdu
				  ){ // set up conditions for events that have enough obs (fix some urdu problems)
					
					*run the optimal bandwidth selection: Cattaneo and Calonico
					
					rdbwselect mar_`v' mar_`v'_lag0 if stack == `d', ///
						c(`thres') p(2) q(3) kernel(triangular) bwselect(mserd) vce(cluster mar) all
					
					if(`e(h_mserd)' < `e(b_mserd)'){ //if effectively exist a bandwidth
						
						*calculate auxiliary variables
						
						replace bw1 = `e(h_cercomb1)' if stack == `d'
						replace bias1 = `e(b_cercomb1)' if stack == `d'
						
					} // end of if statement of existance of bandwidth
				
				} // end of if statament of existance of rd for each stack

			} // end of loop over stacks for rd procedure
			
			* variables for rd samples	
			
			gen rd1 = (mar_`v'_lag0 >= `thres' - bw1 & mar_`v'_lag0 <= `thres' + bw1 & !missing(bw1)) 
			gen biasrd1 = (mar_`v'_lag0 >= `thres' - bias1 & mar_`v'_lag0 <= `thres' + bias1 & !missing(bw1))

			
		} //  end if statement for all except student attendance
		else{ // because student attendance threshold is not constant, need to recenter to 0	
		
			/*for student attendance the thresholds change in time, then we have to normalize 
			threshold to 0*/
			
			preserve
			
			*keep event mar relative time and variable of interest
			
			keep stack mar relativetime mar_`v' red_`v'
			
			/*for the rd we need to compare the flagging period (relative time 0) with the period of 
			the punishment (relative time 1). Then we create the lag of the markaz variable for 1, 
			to have both values of 1 and 0 in the  same row*/
			
			gen temp = mar_`v' if relativetime == 0
			bys stack mar (relativetime): egen mar_`v'_lag0 = max(temp) // period 0 outcome
			drop temp
			
			*keep only the iterest relative time
			
			keep if relativetime == 1 // so compare outcome in 1 with just created outcome in 0
			drop relativetime
			
			*create variables to populate
			
			gen bw1 = .
			gen bias1 = .

			* need to center as in satt thresholds are not constant
			
			gen c_mar_`v' = mar_`v' - red_`v'
			gen c_mar_`v'_lag0 = mar_`v'_lag0 - red_`v'
			
			* estimate rd: we have to iterate over each event dataset
			
			levelsof stack // all possible event-dates 
			foreach d in `r(levels)'{

				qui sum c_mar_`v'_lag0 if stack == `d'
				if(`r(min)' < -2 &  `r(N)' > 50 ){ // from 0 to -2 to avoid conformability error
					
					*run the optimal bandwidth selection: Cattaneo and Calonico
					
					rdbwselect c_mar_`v' c_mar_`v'_lag0 if stack == `d', ///
						c(0) p(2) q(3) kernel(triangular) bwselect(mserd) vce(cluster mar) all
						
					if(`e(h_mserd)' < `e(b_mserd)'){
						
						*calculate auxiliary variables
						replace bw1 = `e(h_cercomb1)' if stack == `d'
						replace bias1 = `e(b_cercomb1)' if stack == `d'
						
					} // end of if statement of existance of bandwidth
					
				} // end of if statament of existance of rd for each stack

			} // end of loop over stacks for rd procedure
			
			*variables for samples	
			
			gen rd1 = (c_mar_`v'_lag0 >= 0 - bw1 & c_mar_`v'_lag0 <= 0 + bw1 & !missing(bw1)) 
			gen biasrd1 = (c_mar_`v'_lag0 >= 0 - bias1 & c_mar_`v'_lag0 <= 0 + bias1 & !missing(bw1))
		
		} //  end else statement for student attendance
		
		* save tempfile
		
		keep stack mar rd1 biasrd1 bw1 bias1
		tempfile rd_sample
		save `rd_sample'
		restore //note the preserve for this restore is within the rd procedure
		
		*merge it with the original stacked data
		
		merge m:1 stack mar using `rd_sample'
		
		* keep desired observations
		
		keep if _merge == 3
		drop _merge

		* merge with original: all schools per mar and date

		joinby mar date_treat using "$analysis/processed/cc_datapacks.dta"

		* mean teachers per markaz
		gen temp = totalteachers if relativetime == 0 // usually we use pre-zero total teachers, as we want to replicate "ignoring pre-values", we use 0
		bys stack mar: egen avg_totalreportedteachers_mar = mean(temp) 
		drop temp

		* save
	
		save "$analysis/processed/stacked/cc_stackdata`i'_`v'_immediate.dta", replace // i = post period, v = outcome
	
		display("finish variable `v' with post periods `i'")

	} // end loop statement for post period i
	
	*after the last `i' post period, it starts again for the next variable v
	display("finish variable `v', pasing to the next one")

} // end loop statement for all variables

*---------------------------------------------------------------------------------------------------
**# 4. markaz orange flagging stacking #

*iterate over each outcome

//1. create event panel
//2. create and adjust event-panel: threshold sample, additionales variables

/*note: for english scores there is not enough sample for RD or full, so the sample size is 
significantly smaller compared with the other scores*/

local treatments tpres satt fac eng math urdu // all flagging outcomes 
foreach v of local treatments{
	
	*iterate over all possible post-periods
	
	forvalues i = 5(1)5{ 
		
		* trick for later create empty temporary file to apend all other evet-panels
		 
		clear		
		gen stack = 0
		tempfile stack
		save `stack'
		
		* main data: balanced dataset
		
		use "$analysis/processed/cc_datapacks.dta", clear
		drop if wing == 3 // we do not consider secondary schools (not subject to monitoring)
		
		/* create time id: date version has gaps between may and september. The new variable takes 
		continuous values since 1, hence, there is no gap
		*/
		
		egen date_treat_e = group(date_treat) // date indicator for iterations

		* validate mar with double assignment of flagging		
		
		gen copy = or_treat_`v'_lead // create a copy of the original variable
		bys mar date_treat: egen templead = nvals(or_treat_`v'_lead) //some obs with double accounting
		bys mar date_treat: egen temp = nvals(or_treat_`v'_) // original date is correct
		bys mar (date_treat `v'): gen t = or_treat_`v'_[_n-1] // update flagging
		bys mar date_treat (`v'): gen tord = _n // set order based on `v'
		replace t = . if tord != 1 // set to missing all except the immediatly before observation for each mar
		bys mar date_treat: egen tmax = max(t) // generalize flagging
		replace or_treat_`v'_lead = tmax if templead == 2 // apply only to those needed to be adjusted
		replace or_treat_`v'_lead = copy if templead == 2 & or_treat_`v'_lead == . // missing replace with original
		drop templead temp t tord tmax copy

		* still double accounting, basically from schools changing mar into a new mar
		* define as the majority of schools reporting (less than 1% so no sensitive)
		bys mar date_treat: egen templead = nvals(or_treat_`v'_lead) //some obs with double accounting
			*count ones and zeroes
		gen zero = 1 if templead == 2 & or_treat_`v'_lead == 0
		gen ones = 1 if templead == 2 & or_treat_`v'_lead == 1
		bys mar date_treat: egen numzeros = sum(zero) if templead == 2
		bys mar date_treat: egen numones = sum(ones) if templead == 2
		replace or_treat_`v'_lead = 0 if numzeros > numones & templead == 2
		replace or_treat_`v'_lead = 1 if numzeros <= numones & templead == 2
		
		drop zero ones numzeros numones templead

		* keep unique mar and date_treat

		*duplicates drop mar date_treat, force // this way of dropping duplicates might be unstable
		* stable solution to drop duplicates
		sort mar date_treat treat_`v'_lead
		gen random = runiform() // generate random variable (note the seed in 0_setup)
		sort mar date_treat treat_`v'_lead random
        bys mar date_treat: gen ord = _n // ordering
		keep if ord == 1
		drop random ord		
		
		keep mar date_treat date_treat_e mar_`v' or_treat_`v'_lead green_`v'

		* define dates at which a treatment will be stackedators by which 

		gen treat_dates = date_treat_e if ((or_treat_`v'_lead == 1) & date_treat > tm(2011m12)) // 2011m12 first flag, but we are using as event the lead

		/* we define datasets for each date where a flag ocurred, so the flag date is the center of 
		each event-data. Note also that date_treat_e correspond to the relative time 1 (punish), 
		then we always will have date_treat_e - 5 to have preperiods (-1, -2, -3, -4) and 0 (flag). 
		We are only changing t based on the post periods, where the minimum is date_treat_e + 1 
		*/

		display("starting variable `v' time `i'")

		* iterate creation of event-panels 
		levelsof treat_dates
		foreach d in `r(levels)'{ 
			
			preserve
			
			*dataset between t+i to t-5 
			
			keep if ((date_treat_e <= `d' + `i') & (date_treat_e >= `d' - 5)) // keep desired time span
			
			*drop if mar outcome is empty
			
			drop if missing(mar_`v')
			
			*adjust treatment in case of missings
			
			bys mar (date_treat): replace or_treat_`v'_lead = or_treat_`v'_[_n-1] if missing(or_treat_`v'_lead)
			
			*drop units that do not have t and t-1
			
			bys mar: egen num_mar = count(mar) //count number of times mar appear
			gen num_periods = 6 + `i' // markaz that appear consecutively 6 + i periods
			keep if num_mar == num_periods // only mars that appear all the time span
			drop num_periods num_mar

			*conditional if exist data
			
			if(_N > 0){
				
				bys mar (date_treat): gen time_id = _n // create time id
					
				/* we must eliminate mar treated in t-n: in such case, a unit would be treated 
				twice (in t-1 and t) or would be treated only on t-1, in which case would not be 
				a clean control
				*/
				
				gen before = (time_id < 6)  // goes from 1 to 6 + `i', so always drop treated in < 6
				sort mar date_treat // sort by date within mar
				bys mar (date_treat): egen treated_before = max(or_treat_`v'_lead) if (before == 1) // treated before time 6
				bys mar: egen max_before = max(treated_before)
				drop if max_before == 1 // drop if treated before
				drop treated_before max_before
					
				*if there still exist data: dropping treated before might lead to no-data for event
				if(_N > 0){
						
					/*if treated in t+1 but not in t, we must drop, as would not be treated in 
					the main event data we are centering the event-data
					*/
						
					*treated in t+1
					gen stag1 = ((or_treat_`v'_lead == 1) & (time_id > 6))
						
					*treated in t
					gen stag2 = ((or_treat_`v'_lead == 1) & (time_id == 6))
						
					*generalizing to all units
					
					bys mar: egen sum_stag1 = max(stag1)
					bys mar: egen sum_stag2 = max(stag2)

					*drop not treated in main event
					
					gen drop_units = (sum_stag2 == 0 & sum_stag1 == 1) // not treated in t, but treated in t+1
					drop if drop_units == 1
					drop before stag1 stag2 sum_stag1 sum_stag2 drop_units
						
					*stack unique id
					
					gen stack = `d'
					drop time_id
					bys mar (date_treat): gen time_id = _n // readjust timeid
					
					
				} // end if statement
				else if(_N == 0){ // if there is no data after dropping the not clean controls
						
					*display if no observations
					display "`d' have 0 observations"
						
				} // end else statement 
			
			} // end if statement
			else if(_N == 0){ // if there is no data for the particular event date
				
				*display if no observations
				display "`d' have 0 observations"

			} // end else statement
			
			*append evet-panel datatsets
			append using `stack' // append to original stack! then for next iterations we rewritte
			tempfile stack
			save `stack'
				
			restore // recover original data to create the next event time panel
		
		} // end of loop statement for event panels of outcomes
		
		* use temporary stacked dataset
		
		use `stack', clear
		
		* drop "temporary file stack"

		drop if stack == 0

		* compress for memory purposes

		compress

		* relative time: normalize to 0 the flag period (time_id == 5)
		
		gen relativetime = time_id - 5

		/* create constant treatment variable: if flag_period = 1 in relativetime == 1 to validate, 
		recreate main treatment
		*/
		
		bys stack mar: gen temp_lead = ((or_treat_`v'_lead == 1) & (time_id == 6))
		bys stack mar: egen flagged = max(temp_lead) //if ever flagged, then treated
		drop temp_lead
		drop if missing(flagged) // drop all the observations that do not report data en the flagging period

		* post-periods
		
		//after punish period
		gen postpunish = (relativetime > 0)
		//after flag period
		gen postflag = (relativetime > -1)
 
		/* regression discontinuity samples:
		
		we need a tighter comparisson of treated and control units, then we use an optimal bandwidth 
		to define for each event-dataset the group of markaz that are around the threshold
		*/
		
		*define thresholds: scores have the same threshold and for student attendance need to defined it separately
		
		if("`v'" == "tpres"){
			local thres = 90
			local min = 90
		}
		else if("`v'" == "fac"){
			local thres = 95
			local min = 93 // for functional facilities there is less variation, then being close to 90 is not enough
		}
		else{
			local thres = 80
			local min = 80
		}
		
		*run rd 
		
		if("`v'" != "satt"){
			
			preserve
			
			*keep event mar relative time and variable of interest
			
			keep stack mar relativetime mar_`v'
			
			/*for the rd we need to compare the flagging period (relative time 0) with the period of 
			the punishment (relative time 1). Then we create the lag of the markaz variable for 1, 
			to have both values of 1 and 0 in the  same row*/
			
			gen temp = mar_`v' if relativetime == 0
			bys stack mar (relativetime): egen mar_`v'_lag0 = max(temp) // period 0 outcome
			drop temp
			
			*keep only the iterest relative time
			
			keep if relativetime == 1 // so compare outcome in 1 with just created outcome in 0
			drop relativetime
			
			*create variables to populate
			
			gen bw1 = .
			gen bias1 = .
			
			* estimate rd: we have to iterate over each event dataset
			
			levelsof stack // all possible event-dates 
			foreach d in `r(levels)'{
				
				display `d'
				qui sum mar_`v'_lag0 if stack == `d'
				*conditions to ensure if threshold sample exist

				if((`r(min)' < `min' & `r(N)' > 50 & "`v'" == "tpres"           )     | /// general conditions
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "fac"             )     | /// general conditions
				   (`r(min)' < `min' & `r(N)' > 30 & "`v'" == "eng"  & `d' != 65)     | /// general conditions: less N for eng to ensure "some" threshold
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "math" & `d' != 65)     | /// conditions for math
				   (`r(min)' < `min' & `r(N)' > 50 & "`v'" == "urdu" & `d' != 65)       /// conditions for urdu
				  ){ // set up conditions for events that have enough obs (fix some urdu problems)
					
					*run the optimal bandwidth selection: Cattaneo and Calonico
					
					rdbwselect mar_`v' mar_`v'_lag0 if stack == `d', ///
						c(`thres') p(2) q(3) kernel(triangular) bwselect(mserd) vce(cluster mar) all
					
					if(`e(h_mserd)' < `e(b_mserd)'){ //if effectively exist a bandwidth
						
						*calculate auxiliary variables
						
						replace bw1 = `e(h_cercomb1)' if stack == `d'
						replace bias1 = `e(b_cercomb1)' if stack == `d'
						
					} // end of if statement of existance of bandwidth
				
				} // end of if statament of existance of rd for each stack

			} // end of loop over stacks for rd procedure
			
			* variables for rd samples	
			
			gen rd1 = (mar_`v'_lag0 >= `thres' - bw1 & mar_`v'_lag0 <= `thres' + bw1 & !missing(bw1)) 
			gen biasrd1 = (mar_`v'_lag0 >= `thres' - bias1 & mar_`v'_lag0 <= `thres' + bias1 & !missing(bw1))

			
		} //  end if statement for all except student attendance
		else{ // because student attendance threshold is not constant, need to recenter to 0	
		
			/*for student attendance the thresholds change in time, then we have to normalize 
			threshold to 0*/
			
			preserve
			
			*keep event mar relative time and variable of interest
			
			keep stack mar relativetime mar_`v' green_`v'
			
			/*for the rd we need to compare the flagging period (relative time 0) with the period of 
			the punishment (relative time 1). Then we create the lag of the markaz variable for 1, 
			to have both values of 1 and 0 in the  same row*/
			
			gen temp = mar_`v' if relativetime == 0
			bys stack mar (relativetime): egen mar_`v'_lag0 = max(temp) // period 0 outcome
			drop temp
			
			*keep only the iterest relative time
			
			keep if relativetime == 1 // so compare outcome in 1 with just created outcome in 0
			drop relativetime
			
			*create variables to populate
			
			gen bw1 = .
			gen bias1 = .

			* need to center as in satt thresholds are not constant
			
			gen c_mar_`v' = mar_`v' - green_`v'
			gen c_mar_`v'_lag0 = mar_`v'_lag0 - green_`v'
			
			* estimate rd: we have to iterate over each event dataset
			
			levelsof stack // all possible event-dates 
			foreach d in `r(levels)'{

				qui sum c_mar_`v'_lag0 if stack == `d'
				if(`r(min)' < -2 &  `r(N)' > 50 ){ // from 0 to -2 to avoid conformability error
					
					*run the optimal bandwidth selection: Cattaneo and Calonico
					
					rdbwselect c_mar_`v' c_mar_`v'_lag0 if stack == `d', ///
						c(0) p(2) q(3) kernel(triangular) bwselect(mserd) vce(cluster mar) all
						
					if(`e(h_mserd)' < `e(b_mserd)'){
						
						*calculate auxiliary variables
						replace bw1 = `e(h_cercomb1)' if stack == `d'
						replace bias1 = `e(b_cercomb1)' if stack == `d'
						
					} // end of if statement of existance of bandwidth
					
				} // end of if statament of existance of rd for each stack

			} // end of loop over stacks for rd procedure
			
			*variables for samples	
			
			gen rd1 = (c_mar_`v'_lag0 >= 0 - bw1 & c_mar_`v'_lag0 <= 0 + bw1 & !missing(bw1)) 
			gen biasrd1 = (c_mar_`v'_lag0 >= 0 - bias1 & c_mar_`v'_lag0 <= 0 + bias1 & !missing(bw1))
		
		} //  end else statement for student attendance
		
		* save tempfile
		
		keep stack mar rd1 biasrd1 bw1 bias1
		tempfile rd_sample
		save `rd_sample'
		restore //note the preserve for this restore is within the rd procedure
		
		*merge it with the original stacked data
		
		merge m:1 stack mar using `rd_sample'
		
		* keep desired observations
		
		keep if _merge == 3
		drop _merge

		* merge with original: all schools per mar and date
		joinby mar date_treat using "$analysis/processed/cc_datapacks.dta"

		* mean teachers per markaz
		gen temp = totalteachers if relativetime < 0
		bys stack mar: egen avg_totalreportedteachers_mar = mean(temp) 
		drop temp

		* save
	
		save "$analysis/processed/stacked/cc_stackdata`i'_`v'_orange.dta", replace // i = post period, v = outcome
	
		display("finish variable `v' with post periods `i'")

	} // end loop statement for post period i
	
	*after the last `i' post period, it starts again for the next variable v
	display("finish variable `v', pasing to the next one")

} // end loop statement for all variables

*---------------------------------------------------------------------------------------------------
**# 5. tehsil red flagging stacking #

*iterate over each outcome

//1. create event panel
//2. create and adjust event-panel: threshold sample, additionales variables

/*to ensure enough variation we were using 50 markaz minimum per stack. Because we are now using 
tehsils, we need to use an smaller sample size per markaz to define flagged tehsils around the 
threshold and some specific tweking by variable to ensure it works */

local treatments tpres satt fac eng math urdu // all flagging outcomes
foreach v of local treatments{
	
	*iterate over all possible post-periods
	
	forvalues i = 5(1)5{ 
		
		* trick for later create empty temporary file to apend all other evet-panels
		
		clear		
		gen stack = 0
		tempfile stack
		save `stack'
		
		* main data: balanced dataset
		
		use "$analysis/processed/cc_datapacks.dta", clear
		drop if wing == 3 // we do not consider secondary schools (not subject to monitoring)
		/* create time id: date version has gaps between may and september. The new variable takes 
		continuous values since 1, hence, there is no gap
		*/
		egen date_treat_e = group(date_treat) // date indicator for iterations

		* validate mar with double assignment of flagging		
		gen copy = tehsil_treat_`v'_lead // create a copy of the original variable
		bys tehsilwing date_treat: egen templead = nvals(tehsil_treat_`v'_lead) //some obs with double accounting
		bys tehsilwing date_treat: egen temp = nvals(tehsil_treat_`v'_) // original date also have doubles
		
		*fix original date
		* define as the majority of schools reporting (less than 1% so no sensitive)
			*count ones and zeroes
		gen zero = 1 if temp == 2 & tehsil_treat_`v'_ == 0
		gen ones = 1 if temp == 2 & tehsil_treat_`v'_ == 1
		bys tehsilwing date_treat: egen numzeros = sum(zero) if temp == 2
		bys tehsilwing date_treat: egen numones = sum(ones) if temp == 2
		replace tehsil_treat_`v'_ = 0 if numzeros > numones & temp == 2
		replace tehsil_treat_`v'_ = 1 if numzeros <= numones & temp == 2
		
		drop zero ones numzeros numones
		
		* fix lead flagging
		
		bys tehsilwing (date_treat `v'): gen t = tehsil_treat_`v'_[_n-1] // update flagging
		bys tehsilwing date_treat (`v'): gen tord = _n // set order based on `v'
		replace t = . if tord != 1 // set to missing all except the immediatly before observation for each mar
		bys tehsilwing date_treat: egen tmax = max(t) // generalize flagging
		replace tehsil_treat_`v'_lead = tmax if templead == 2 // apply only to those needed to be adjusted
		replace tehsil_treat_`v'_lead = copy if templead == 2 & tehsil_treat_`v'_lead == . // missing replace with original
		drop templead temp t tord tmax copy

		* still double accounting, basically from schools changing mar into a new mar
		* define as the majority of schools reporting (less than 1% so no sensitive)
		bys tehsilwing date_treat: egen templead = nvals(tehsil_treat_`v'_lead) //some obs with double accounting
			*count ones and zeroes
		gen zero = 1 if templead == 2 & tehsil_treat_`v'_lead == 0
		gen ones = 1 if templead == 2 & tehsil_treat_`v'_lead == 1
		bys tehsilwing date_treat: egen numzeros = sum(zero) if templead == 2
		bys tehsilwing date_treat: egen numones = sum(ones) if templead == 2
		replace tehsil_treat_`v'_lead = 0 if numzeros > numones & templead == 2
		replace tehsil_treat_`v'_lead = 1 if numzeros <= numones & templead == 2
		
		drop zero ones numzeros numones templead
		
		/* one additional thing making results unstable: tehsil might have 
		multiple values */
		bys tehsilwing date_treat: egen temp = nvals(tehsil_`v')
		bys tehsilwing date_treat: egen t = mean(tehsil_`v') if temp > 1
		replace tehsil_`v' = t if temp > 1
		
		drop temp t 
		
		* last, standardize thresholds!
		* mainly for tehsil, where markaz have different red threshold, it might be problematic
		bys tehsilwing: egen temp = mean(red_`v')
		replace red_`v' = temp
		drop temp

		* keep unique mar and date_treat

		*duplicates drop mar date_treat, force // this way of dropping duplicates might be unstable
		* stable solution to drop duplicates
		sort tehsilwing date_treat tehsil_treat_`v'_lead tehsil_`v' `v'
		gen random = runiform() // generate random variable (note the seed in 0_setup)
		sort tehsilwing date_treat tehsil_treat_`v'_lead tehsil_`v' `v' random
        bys tehsilwing date_treat (tehsil_treat_`v'_lead tehsil_`v' `v' random): gen ord = _n // ordering
		keep if ord == 1
		drop random ord		
				
		keep tehsilwing date_treat date_treat_e tehsil_`v' tehsil_treat_`v'_lead red_`v'

		* define dates at which a treatment will be stackedators by which 

		gen treat_dates = date_treat_e if ((tehsil_treat_`v'_lead == 1) & date_treat > tm(2011m12)) // 2011m12 first flag, but we are using as event the lead

		/* we define datasets for each date where a flag ocurred, so the flag date is the center of 
		each event-data. Note also that date_treat_e correspond to the relative time 1 (punish), 
		then we always will have date_treat_e - 5 to have preperiods (-1, -2, -3, -4) and 0 (flag). 
		We are only changing t based on the post periods, where the minimum is date_treat_e + 1 
		*/

		display("starting variable `v' time `i'")

		* iterate creation of event-panels 
		levelsof treat_dates
		foreach d in `r(levels)'{ 
			
			preserve
			
			*dataset between t+i to t-5 
			
			keep if ((date_treat_e <= `d' + `i') & (date_treat_e >= `d' - 5)) // keep desired time span
			
			*drop if mar outcome is empty
			
			drop if missing(tehsil_`v')
			
			*adjust treatment in case of missings
			
			bys tehsilwing (date_treat): replace tehsil_treat_`v'_lead = tehsil_treat_`v'_[_n-1] if missing(tehsil_treat_`v'_lead)
			
			*drop units that do not have t and t-1
			
			bys tehsilwing: egen num_tehsil = count(tehsilwing) //count number of times mar appear
			gen num_periods = 6 + `i' // markaz that appear consecutively 6 + i periods
			keep if num_tehsil == num_periods // only mars that appear all the time span
			drop num_periods num_tehsil

			*conditional if exist data
			
			if(_N > 0){
				
				bys tehsilwing (date_treat): gen time_id = _n // create time id
					
				/* we must eliminate mar treated in t-n: in such case, a unit would be treated 
				twice (in t-1 and t) or would be treated only on t-1, in which case would not be 
				a clean control
				*/
				
				gen before = (time_id < 6)  // goes from 1 to 6 + `i', so always drop treated in < 6
				sort tehsilwing date_treat // sort by date within mar
				bys tehsilwing (date_treat): egen treated_before = max(tehsil_treat_`v'_lead) if (before == 1) // treated before time 6
				bys tehsilwing: egen max_before = max(treated_before)
				drop if max_before == 1 // drop if treated before
				drop treated_before max_before
					
				*if there still exist data: dropping treated before might lead to no-data for event
				if(_N > 0){
						
					/*if treated in t+1 but not in t, we must drop, as would not be treated in 
					the main event data we are centering the event-data
					*/
						
					*treated in t+1
					gen stag1 = ((tehsil_treat_`v'_lead == 1) & (time_id > 6))
						
					*treated in t
					gen stag2 = ((tehsil_treat_`v'_lead == 1) & (time_id == 6))
						
					*generalizing to all units
					
					bys tehsilwing: egen sum_stag1 = max(stag1)
					bys tehsilwing: egen sum_stag2 = max(stag2)

					*drop not treated in main event
					
					gen drop_units = (sum_stag2 == 0 & sum_stag1 == 1) // not treated in t, but treated in t+1
					drop if drop_units == 1
					drop before stag1 stag2 sum_stag1 sum_stag2 drop_units
						
					*stack unique id
					
					gen stack = `d'
					drop time_id
					bys tehsilwing (date_treat): gen time_id = _n // readjust timeid
					
					
				} // end if statement
				else if(_N == 0){ // if there is no data after dropping the not clean controls
						
					*display if no observations
					display "`d' have 0 observations"
						
				} // end else statement 
			
			} // end if statement
			else if(_N == 0){ // if there is no data for the particular event date
				
				*display if no observations
				display "`d' have 0 observations"

			} // end else statement
			
			*append evet-panel datatsets
			append using `stack' // append to original stack! then for next iterations we rewritte
			tempfile stack
			save `stack'
				
			restore // recover original data to create the next event time panel
		
		} // end of loop statement for event panels of outcomes
		
		* use temporary stacked dataset
		
		use `stack', clear

		* drop "temporary file stack"

		drop if stack == 0

		* compress for memory purposes

		compress

		* relative time: normalize to 0 the flag period (time_id == 5)
		
		gen relativetime = time_id - 5

		/* create constant treatment variable: if flag_period = 1 in relativetime == 1 to validate, 
		recreate main treatment
		*/
		
		bys stack tehsilwing: gen temp_lead = ((tehsil_treat_`v'_lead == 1) & (time_id == 6))
		bys stack tehsilwing: egen flagged = max(temp_lead) //if ever flagged, then treated
		drop temp_lead
		drop if missing(flagged) // drop all the observations that do not report data en the flagging period

		* post-periods
		
		//after punish period
		gen postpunish = (relativetime > 0)
		//after flag period
		gen postflag = (relativetime > -1)
 
		/* regression discontinuity samples:
		
		we need a tighter comparisson of treated and control units, then we use an optimal bandwidth 
		to define for each event-dataset the group of markaz that are around the threshold
		*/
		
		*define thresholds: scores have the same threshold and for student attendance need to defined it separately
		
		if("`v'" == "tpres"){
			local thres = 86
			local min = 84
		}
		else if("`v'" == "fac"){
			local thres = 90
			local min = 87 // for functional facilities there is less variation, then being close to 90 is not enough
		}
		else{
			local thres = 70
			local min = 68
		}
		
		*run rd 
		
		if("`v'" == "tpres" | "`v'" == "fac"){
			
			preserve
			
			*keep event mar relative time and variable of interest
			
			keep stack tehsilwing relativetime tehsil_`v'
			
			/*for the rd we need to compare the flagging period (relative time 0) with the period of 
			the punishment (relative time 1). Then we create the lag of the markaz variable for 1, 
			to have both values of 1 and 0 in the  same row*/
			
			gen temp = tehsil_`v' if relativetime == 0
			bys stack tehsilwing (relativetime): egen tehsil_`v'_lag0 = max(temp) // period 0 outcome
			drop temp
			
			*keep only the iterest relative time
			
			keep if relativetime == 1 // so compare outcome in 1 with just created outcome in 0
			drop relativetime
			
			*create variables to populate
			
			gen bw1 = .
			gen bias1 = .
			
			* estimate rd: we have to iterate over each event dataset
			
			levelsof stack // all possible event-dates 
			foreach d in `r(levels)'{
				
				display `d'
				qui sum tehsil_`v'_lag0 if stack == `d'
				
				*conditions to ensure if threshold sample exist
				if((`r(min)' < `min' & `r(N)' > 10 & "`v'" == "tpres" & `d' != 47 & `d' != 23) | /// general conditions - for stack 23 there is a bug with rdbwselect!
				   (`r(min)' < `min' & `r(N)' > 15 & "`v'" == "fac"   & `d' != 20 & `d' != 22) /// general conditions: specific for fac
				  ){ // set up conditions for events that have enough obs (fix some urdu problems)
					
					*run the optimal bandwidth selection: Cattaneo and Calonico
					
					rdbwselect tehsil_`v' tehsil_`v'_lag0 if stack == `d', ///
						c(`thres') p(2) q(3) kernel(triangular) bwselect(mserd) vce(cluster tehsilwing) all
					
					if(`e(h_mserd)' < `e(b_mserd)'){ //if effectively exist a bandwidth
						
						*calculate auxiliary variables
						
						replace bw1 = `e(h_cercomb1)' if stack == `d'
						replace bias1 = `e(b_cercomb1)' if stack == `d'
						
					} // end of if statement of existance of bandwidth
				
				} // end of if statament of existance of rd for each stack

			} // end of loop over stacks for rd procedure
			
			* variables for rd samples	
			
			gen rd1 = (tehsil_`v'_lag0 >= `thres' - bw1 & tehsil_`v'_lag0 <= `thres' + bw1 & !missing(bw1)) 
			gen biasrd1 = (tehsil_`v'_lag0 >= `thres' - bias1 & tehsil_`v'_lag0 <= `thres' + bias1 & !missing(bw1))

			
		} //  end if statement for all except student attendance
		else if("`v'" == "eng" | "`v'" == "math" | "`v'" == "urdu"){ // low number of observations, use different bandwidth definition
			
			preserve
			
			*keep event mar relative time and variable of interest
			
			keep stack tehsilwing relativetime tehsil_`v'
			
			/*for the rd we need to compare the flagging period (relative time 0) with the period of 
			the punishment (relative time 1). Then we create the lag of the markaz variable for 1, 
			to have both values of 1 and 0 in the  same row*/
			
			gen temp = tehsil_`v' if relativetime == 0
			bys stack tehsilwing (relativetime): egen tehsil_`v'_lag0 = max(temp) // period 0 outcome
			drop temp
			
			*keep only the iterest relative time
			
			keep if relativetime == 1 // so compare outcome in 1 with just created outcome in 0
			drop relativetime
			
			*create variables to populate
			
			gen bw1 = .
			gen bw2 = .
			gen bias1 = .
			gen bias2 = .
			
			* estimate rd: we have to iterate over each event dataset
			
			levelsof stack // all possible event-dates 
			foreach d in `r(levels)'{
				
				display `d'
				qui sum tehsil_`v'_lag0 if stack == `d'
				if((`r(min)' < `min' & `r(N)' > 10 & "`v'" == "eng"  & `d' != 65)     | /// general conditions
				   (`r(min)' < `min' & `r(N)' > 10 & "`v'" == "math" & `d' != 61 & `d' != 65)     | /// conditions for math
				   (`r(min)' < `min' & `r(N)' > 10 & "`v'" == "urdu" & `d' != 50 & `d' != 65)       /// conditions for urdu
				  ){ // set up conditions for events that have enough obs (fix some urdu problems)
					
					*run the optimal bandwidth selection: Cattaneo and Calonico
					
					rdbwselect tehsil_`v' tehsil_`v'_lag0 if stack == `d', ///
						c(`thres') p(2) q(3) kernel(triangular) bwselect(msetwo) vce(cluster tehsilwing) all
					
					*calculate auxiliary variables
					*usually test that bias is larger than bw, but as different bw each side, then assume it exist 
						
					replace bw1 = `e(h_certwo_l)' if stack == `d'
					replace bw2 = `e(h_certwo_r)' if stack == `d'
					replace bias1 = `e(b_certwo_l)' if stack == `d'
					replace bias2 = `e(b_certwo_r)' if stack == `d'
						
				
				} // end of if statament of existance of rd for each stack

			} // end of loop over stacks for rd procedure
			
			* variables for rd samples	
			
			gen rd1 = (tehsil_`v'_lag0 >= `thres' - bw1 & tehsil_`v'_lag0 <= `thres' + bw2 & !missing(bw1)) 
			gen biasrd1 = (tehsil_`v'_lag0 >= `thres' - bias1 & tehsil_`v'_lag0 <= `thres' + bias2 & !missing(bw1))

			
		} //  end if statement for all except student attendance
		else{ // because student attendance threshold is not constant, need to recenter to 0	
		
			/*for student attendance the thresholds change in time, then we have to normalize 
			threshold to 0*/
			
			preserve
			
			*keep event mar relative time and variable of interest
			
			keep stack tehsilwing relativetime tehsil_`v' red_`v'
			
			/*for the rd we need to compare the flagging period (relative time 0) with the period of 
			the punishment (relative time 1). Then we create the lag of the markaz variable for 1, 
			to have both values of 1 and 0 in the  same row*/
			
			gen temp = tehsil_`v' if relativetime == 0
			bys stack tehsilwing (relativetime): egen tehsil_`v'_lag0 = mean(temp) // period 0 outcome
			drop temp
			
			*keep only the iterest relative time
			
			keep if relativetime == 1 // so compare outcome in 1 with just created outcome in 0
			drop relativetime
			
			*create variables to populate
			
			gen bw1 = .
			gen bias1 = .

			* need to center as in satt thresholds are not constant
			
			gen c_tehsil_`v' = tehsil_`v' - red_`v'
			gen c_tehsil_`v'_lag0 = tehsil_`v'_lag0 - red_`v'
			
			* estimate rd: we have to iterate over each event dataset
			
			levelsof stack // all possible event-dates 
			foreach d in `r(levels)'{
				
				display `d'
				qui sum c_tehsil_`v'_lag0 if stack == `d'
				if(`r(min)' < -3 &  `r(N)' > 10 & `d' != 18){ // from 0 to -2 to avoid conformability error
					
					*run the optimal bandwidth selection: Cattaneo and Calonico
					
					rdbwselect c_tehsil_`v' c_tehsil_`v'_lag0 if stack == `d', ///
						c(0) p(2) q(3) kernel(triangular) bwselect(mserd) vce(cluster tehsilwing) all
						
					if(`e(h_mserd)' < `e(b_mserd)'){
						
						*calculate auxiliary variables
						replace bw1 = `e(h_cercomb1)' if stack == `d'
						replace bias1 = `e(b_cercomb1)' if stack == `d'
						
					} // end of if statement of existance of bandwidth
					
				} // end of if statament of existance of rd for each stack

			} // end of loop over stacks for rd procedure
			
			*variables for samples	
			
			gen rd1 = (c_tehsil_`v'_lag0 >= 0 - bw1 & c_tehsil_`v'_lag0 <= 0 + bw1 & !missing(bw1)) 
			gen biasrd1 = (c_tehsil_`v'_lag0 >= 0 - bias1 & c_tehsil_`v'_lag0 <= 0 + bias1 & !missing(bw1))
		
		} //  end else statement for student attendance
		
		* save tempfile
		
		keep stack tehsilwing rd1 biasrd1 bw1 bias1
		tempfile rd_sample
		save `rd_sample'
		restore //note the preserve for this restore is within the rd procedure
		
		*merge it with the original stacked data
		
		merge m:1 stack tehsilwing using `rd_sample'
		
		* keep desired observations
		
		keep if _merge == 3
		drop _merge

		* merge with original: all schools per mar and date
		
		joinby tehsilwing date_treat using "$analysis/processed/cc_datapacks.dta"

		* mean teachers per markaz
		gen temp = totalteachers if relativetime < 0
		bys stack tehsilwing: egen avg_totalreportedteachers_tehsil = mean(temp) 
		drop temp

		* save
	
		save "$analysis/processed/stacked/cc_stackdata`i'_`v'_tehsil.dta", replace // i = post period, v = outcome
	
		display("finish variable `v' with post periods `i'")

	} // end loop statement for post period i
	
	*after the last `i' post period, it starts again for the next variable v
	display("finish variable `v', pasing to the next one")

} // end loop statement for all variables

*---------------------------------------------------------------------------------------------------
**# 6. district flagging stacking (ranking) #

/* an alternative source of variation in terms of flagging consist in the quarterly stocktakes where 
district officials meet with chief minister. Officers in top districts are rewarded while officers in
bottom are punished. We use this variation as an alternative treatment but using as events quarterly 
meetings (instead of monthly) 

Also, the ranking for defining top/bottom does not depend on each individual outcome but rather in 
aggregate performance, so we only create a top/bottom stacked data (not for each outcome)
*/

local position bottom top
foreach v of local position{
	
	* trick for later create empty temporary file to apend all other evet-panels
		
	clear		
	gen stack = 0
	tempfile stack
	save `stack'
    
	* ranking data
	
	//format of ranking districts
	use "$analysis/data/ranking", replace
	gen quarterly_meet = 1 // date of meeting
	drop district_bottom district_top
	gen district_bottom = (rank_position < 6) // bottom 5
	gen district_top = (rank_position > 31) // top 5
	tempfile rank
	save `rank'

	*merge with balanced data
	
	use "$analysis/processed/cc_datapacks.dta", clear 
	merge m:1 district date_treat using `rank' // merge main with rankings data
	drop _merge

	* keep unique mar and date_treat

	*duplicates drop mar date_treat, force // this way of dropping duplicates might be unstable
	* stable solution to drop duplicates
	sort district date_treat district_`v'
	gen random = runiform() // generate random variable (note the seed in 0_setup)
	sort district date_treat district_`v' random
    bys district date_treat district_`v': gen ord = _n // ordering
	keep if ord == 1
	drop random ord

	/* create time id: date version has gaps between may and september. The new variable takes 
	continuous values since 1, hence, there is no gap
	*/
	
	egen date_treat_e = group(date_treat)
	
	*define dates at which a treatment appear
	
	gen treat_dates = date_treat_e if district_`v' == 1

	*define datasets for each cohort-event
	
	levelsof treat_dates
	foreach d in `r(levels)'{
		
		preserve
		
		*dataset between t+6 to t-4 (where t == 0, when district meeting happened)
		
		keep if date_treat_e <= `d' + 3 & date_treat_e >= `d' - 4 // fix time window
		
		*drop units that do not have t and t-1

		bys district: egen num_dis = count(district)
		keep if num_dis == 8
		drop num_dis
		
		*conditional if exist data
		
		if(_N > 0){
			
			bys district (date_treat): gen time_id = _n
			
			/* we must eliminate mar treated in t-n: in such case, a unit would be treated 
			twice (in t-1 and t) or would be treated only on t-1, in which case would not be 
			a clean control
			*/
			
			gen before = (time_id < 5) // here there is no dip
			sort district date_treat
			bys district (date_treat): egen treated_before = max(district_`v') if (before == 1)
			bys district: egen max_before = max(treated_before)
			drop if max_before == 1 // drop if treated before
			drop treated_before max_before
			
			*treated in t+1
			
			gen stag1 = (district_`v' == 1 & time_id > 5)
			
			*treated in t
			
			gen stag2 = (district_`v' == 1 & time_id == 5)
			
			*generalizing to all units
			
			bys district: egen sum_stag1 = max(stag1)
			bys district: egen sum_stag2 = max(stag2)
			
			*note that if treated in t+1 but not in t, we must drop
			
			gen drop_units = (sum_stag2 == 0 & sum_stag1 == 1)
			drop if drop_units == 1
			drop before stag1 stag2 sum_stag1 sum_stag2 drop_units
			
			*stack unique id
			
			gen stack = `d'
			
			*replace missings with 0: note apart from the treat date, the rest are 0 (no flagged)
			
			replace district_`v' = 0 if missing(district_`v')
			drop time_id
			bys district (date_treat): gen time_id = _n
			
			*merging
			
			keep district district_`v' stack date_treat rank_position time_id
			joinby district date_treat using "$analysis/processed/cc_datapacks.dta"
			*joinby district date_treat using "$analysis/processed/cc_datapacks.dta", _merge(_merge)
			*keep if _merge == 3
			*drop _merge
			
		} // end if statement for data availability
		else if(_N == 0){
			
			*display if no observations
			
			display "`d' have 0 observations"
			
		} // end of else statement if no data
					
		*append evet-panel datatsets
		
		append using `stack' // append to original stack! then for next iterations we rewritte
		tempfile stack
		save `stack'
		
		restore
		
	} // end of stacking by dates
		
	* use temporary stacked dataset
		
	use `stack', clear
	
	* drop "temporary file stack"

	drop if stack == 0

	* drop non valid observations

	drop if wing == 3
	merge 1:1 stack district emis date_treat using "$analysis/data/cc_valid_districtemis_`v'"
	keep if _merge == 3
	drop _merge

	* compress for memory purposes

	compress

	*relative time: normalize to 0 the measure period
	
	gen relativetime = time_id - 5

	*create constant treatment variable: if flag_period = 1 in relativetime == 0 (time_id == 5)
	
	bys stack district: gen temp_lead = (district_`v' == 1 & time_id == 5)
	bys stack district: egen flagged = max(temp_lead) //if ever flagged, then treated
	drop temp_lead
	drop if flagged == . //i drop all the observations that do not report data en the flagging period

	/*corrected sample
		
	note that we cannot formulate an RD sample for the district flagging as it is based on a 
	ranking, nevertheless we can compare the districts near the bottom/top 5 (flagged districts)
	*/
		
	gen temp = rank_position if time_id == 5
	bys stack district: egen rank = max(temp)
	drop temp
	gen rd_bottom = (rank <= 10)
	gen rd_top = (rank >= 27)
	
	*average district level teachers
	
	bys stack district mar : egen temp = sum(totalteachers) 
	gen temp_mar = temp if relativetime < 0
	bys stack district: egen avg_totalreportedteachers_dis = mean(temp_mar) 
	drop temp_mar
	
	*saving
	
	save "$analysis/processed/stacked/cc_stackdata_`v'_district.dta", replace // v for top/bottom

}

























